]> git.sesse.net Git - movit/blobdiff - effect.h
Change to using GLEW everywhere.
[movit] / effect.h
index ef58392715ea7ade816bfc440c1eec65bd3ee410..235558f8e1c4cf9f480361f0f428916885b1e0e5 100644 (file)
--- a/effect.h
+++ b/effect.h
 
 #include <assert.h>
 
-#include "opengl.h"
+#include <Eigen/Core>
+
+#include <GL/glew.h>
+#include "util.h"
 
 class EffectChain;
 class Node;
@@ -41,15 +44,17 @@ struct RGBTriplet {
 GLint get_uniform_location(GLuint glsl_program_num, const std::string &prefix, const std::string &key);
 void set_uniform_int(GLuint glsl_program_num, const std::string &prefix, const std::string &key, int value);
 void set_uniform_float(GLuint glsl_program_num, const std::string &prefix, const std::string &key, float value);
-void set_uniform_float_array(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const float *values, size_t num_values);
 void set_uniform_vec2(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const float *values);
 void set_uniform_vec3(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const float *values);
 void set_uniform_vec4_array(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const float *values, size_t num_values);
+void set_uniform_mat3(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const Eigen::Matrix3d &matrix);
 
 class Effect {
 public:
+       virtual ~Effect() {}
+
        // An identifier for this type of effect, mostly used for debug output
-       // (but some special names, like "ColorSpaceConversionEffect", holds special
+       // (but some special names, like "ColorspaceConversionEffect", holds special
        // meaning). Same as the class name is fine.
        virtual std::string effect_type_id() const = 0;
 
@@ -87,7 +92,7 @@ public:
        //  2. You rely on previous effects, possibly including gamma
        //     expansion, to happen pre-filtering instead of post-filtering.
        //     (This is only relevant if you actually need the filtering; if
-       //     you sample on whole input pixels only, it makes no difference.)
+       //     you sample 1:1 between pixels and texels, it makes no difference.)
        //
        // Note that in some cases, you might get post-filtered gamma expansion
        // even when setting this option. More specifically, if you are the
@@ -105,9 +110,9 @@ public:
        virtual bool needs_mipmaps() const { return false; }
 
        // Whether this effect wants to output to a different size than
// its input(s). If you set this to true, the output will be
// bounced to a texture (similarly to if the next effect set
// needs_texture_bounce()).
        virtual bool changes_output_size() const { return false; }
 
        // If changes_output_size() is true, you must implement this to tell
@@ -119,6 +124,18 @@ public:
                assert(false);
        }
 
+       // Tells the effect the resolution of each of its input.
+       // This will be called every frame, and always before get_output_size(),
+       // so you can change your output size based on the input if so desired.
+       //
+       // Note that in some cases, an input might not have a single well-defined
+       // resolution (for instance if you fade between two inputs with
+       // different resolutions). In this case, you will get width=0 and height=0
+       // for that input. If you cannot handle that, you will need to set
+       // needs_texture_bounce() to true, which will force a render to a single
+       // given resolution before you get the input.
+       virtual void inform_input_size(unsigned input_num, unsigned width, unsigned height) {}
+
        // How many inputs this effect will take (a fixed number).
        // If you have only one input, it will be called INPUT() in GLSL;
        // if you have several, they will be INPUT1(), INPUT2(), and so on.
@@ -162,10 +179,10 @@ public:
 
        // Set a parameter; intended to be called from user code.
        // Neither of these take ownership of the pointer.
-       virtual bool set_int(const std::string&, int value);
-       virtual bool set_float(const std::string &key, float value);
-       virtual bool set_vec2(const std::string &key, const float *values);
-       virtual bool set_vec3(const std::string &key, const float *values);
+       virtual bool set_int(const std::string&, int value) MUST_CHECK_RESULT;
+       virtual bool set_float(const std::string &key, float value) MUST_CHECK_RESULT;
+       virtual bool set_vec2(const std::string &key, const float *values) MUST_CHECK_RESULT;
+       virtual bool set_vec3(const std::string &key, const float *values) MUST_CHECK_RESULT;
 
 protected:
        // Register a parameter. Whenever set_*() is called with the same key,
@@ -105,9 +110,9 @@ public:
 
        // Whether this effect wants to output to a different size than
-       // its input(s). If you set this to true, the output will be
-       // bounced to a texture (similarly to if the next effect set
-       // needs_texture_bounce()).
+       // its input(s) (see inform_input_size(), below). If you set this to
+       // true, the output will be bounced to a texture (similarly to if the
+       // next effect set needs_texture_bounce()).
        virtual bool changes_output_size() const { return false; }
 